@@ -1,6 +1,6 @@
 /*
 PStreams - POSIX Process I/O for C++
-Copyright (C) 2002-2010 Jonathan Wakely
+Copyright (C) 2002-2014 Jonathan Wakely
 
 This file is part of PStreams.
 
@@ -240,7 +240,7 @@ int main()
 
     {
         // test unformatted input
-        // should print hostname on stdout, prefixed by "STDOUT: "
+        // should print date on stdout, prefixed by "STDOUT: "
         ipstream host("date");
         str.clear();
         char c;
@@ -250,6 +250,39 @@ int main()
         print_result(host, host.eof());
     }
 
+    {
+        // test execve() style construction
+        // should print date on stdout, prefixed by "STDOUT: "
+
+        pstreams::argv_type argv;
+        argv.push_back("date");
+        ipstream is1(argv[0], argv);
+        ipstream is2(argv);
+        string s1, s2;
+        getline(is1, s1);
+        getline(is2, s2);
+        cout << "STDOUT:  " << s1 << endl;
+        print_result(is1, !s1.empty());
+        print_result(is2, s1 == s2);
+    }
+
+    {
+        // test execve() style construction
+        // should print error on stdout, prefixed by "STDERR: "
+
+        pstreams::argv_type argv;
+        argv.push_back("cat");
+        argv.push_back("/nosuchdir/nosuchfile");
+        ipstream is1(argv[0], argv, pstreams::pstderr);
+        ipstream is2(argv, pstreams::pstderr);
+        string s1, s2;
+        getline(is1, s1);
+        getline(is2, s2);
+        cout << "STDERR:  " << s1 << endl;
+        print_result(is1, !s1.empty());
+        print_result(is2, s1 == s2);
+    }
+
     {
         // open after construction, then write
         opstream os;
@@ -802,13 +835,13 @@ int main()
             explicit pguard(ipstream& in, int signal=SIGKILL)
             : buf_(*in.rdbuf()), signal_(signal) { }
 
-          ~pguard() { if (signal_) buf_.kill(signal_); }
+            ~pguard() { if (signal_) buf_.kill(signal_); }
 
-          void release() { signal_ = 0; }
+            void release() { signal_ = 0; }
 
         private:
-          pstreambuf& buf_;
-          int signal_;
+            pstreambuf& buf_;
+            int signal_;
         };
 
         in.clear();
@@ -834,6 +867,33 @@ int main()
         }
     }
 
+    clog << "# Testing process groups\n";
+    {
+        pstreams::argv_type args;
+        args.push_back("/bin/sleep");
+        args.push_back("10");
+        ipstream in(args);
+        void* p = in.rdbuf()->killpg(SIGKILL);
+        print_result(in, !p);
+        print_result(in, in.rdbuf()->error() == EPERM);
+        in.rdbuf()->kill(SIGTERM);
+        in.close();
+        int status = in.rdbuf()->status();
+        print_result(in, WIFSIGNALED(status) && WTERMSIG(status)==SIGTERM);
+    }
+    {
+        pstreams::argv_type args;
+        args.push_back("/bin/sleep");
+        args.push_back("10");
+        ipstream in(args, pstreams::newpg);
+        void* p = in.rdbuf()->killpg(SIGKILL);
+        print_result(in, p);
+        print_result(in, in.rdbuf()->error() == 0);
+        in.close();
+        int status = in.rdbuf()->status();
+        print_result(in, WIFSIGNALED(status) && WTERMSIG(status)==SIGKILL);
+    }
+
     clog << "# Testing wide chars\n";
     {
         ipstream dummy("true");
